@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,13 +13,11 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 from collections import defaultdict
 
 import attr
 
-from hypothesis.internal.compat import hbytes, hrange, int_from_bytes, int_to_bytes
+from hypothesis.internal.compat import int_from_bytes, int_to_bytes
 from hypothesis.internal.conjecture.choicetree import ChoiceTree
 from hypothesis.internal.conjecture.data import ConjectureResult, Overrun, Status
 from hypothesis.internal.conjecture.floats import (
@@ -67,7 +63,7 @@ SHRINK_PASS_DEFINITIONS = {}  # type: Dict[str, ShrinkPassDefinition]
 
 
 @attr.s()
-class ShrinkPassDefinition(object):
+class ShrinkPassDefinition:
     """A shrink pass bundles together a large number of local changes to
     the current shrink target.
 
@@ -109,7 +105,7 @@ def defines_shrink_pass():
     return accept
 
 
-class Shrinker(object):
+class Shrinker:
     """A shrinker is a child object of a ConjectureRunner which is designed to
     manage the associated state of a particular shrink problem. That is, we
     have some initial ConjectureData object and some property of interest
@@ -262,7 +258,7 @@ class Shrinker(object):
         accept.__name__ = fn.__name__
         return property(accept)
 
-    def __init__(self, engine, initial, predicate):
+    def __init__(self, engine, initial, predicate, allow_transition):
         """Create a shrinker for a particular engine, with a given starting
         point and predicate. When shrink() is called it will attempt to find an
         example for which predicate is True and which is strictly smaller than
@@ -271,8 +267,10 @@ class Shrinker(object):
         Note that initial is a ConjectureData object, and predicate
         takes ConjectureData objects.
         """
+        assert predicate is not None or allow_transition is not None
         self.engine = engine
-        self.__predicate = predicate
+        self.__predicate = predicate or (lambda data: True)
+        self.__allow_transition = allow_transition or (lambda source, destination: True)
         self.__derived_values = {}
         self.__pending_shrink_explanation = None
 
@@ -340,7 +338,7 @@ class Shrinker(object):
     def consider_new_buffer(self, buffer):
         """Returns True if after running this buffer the result would be
         the current shrink_target."""
-        buffer = hbytes(buffer)
+        buffer = bytes(buffer)
         return buffer.startswith(self.buffer) or self.incorporate_new_buffer(buffer)
 
     def incorporate_new_buffer(self, buffer):
@@ -348,7 +346,7 @@ class Shrinker(object):
         that changed the shrink_target, or determines that doing so would
         be useless and returns False without running it."""
 
-        buffer = hbytes(buffer[: self.shrink_target.index])
+        buffer = bytes(buffer[: self.shrink_target.index])
         # Sometimes an attempt at lexicographic minimization will do the wrong
         # thing because the buffer has changed under it (e.g. something has
         # turned into a write, the bit size has changed). The result would be
@@ -367,16 +365,15 @@ class Shrinker(object):
 
     def incorporate_test_data(self, data):
         """Takes a ConjectureData or Overrun object updates the current
-        shrink_target if this data represents an improvement over it,
-        returning True if it is."""
-        if data is Overrun or data is self.shrink_target:
+        shrink_target if this data represents an improvement over it."""
+        if data.status < Status.VALID or data is self.shrink_target:
             return
-        if self.__predicate(data) and sort_key(data.buffer) < sort_key(
-            self.shrink_target.buffer
+        if (
+            self.__predicate(data)
+            and sort_key(data.buffer) < sort_key(self.shrink_target.buffer)
+            and self.__allow_transition(self.shrink_target, data)
         ):
             self.update_shrink_target(data)
-            return True
-        return False
 
     def cached_test_function(self, buffer):
         """Returns a cached version of the underlying test function, so
@@ -388,7 +385,7 @@ class Shrinker(object):
             self.debug(self.__pending_shrink_explanation)
             self.__pending_shrink_explanation = None
 
-        buffer = hbytes(buffer)
+        buffer = bytes(buffer)
         result = self.engine.cached_test_function(buffer)
         self.incorporate_test_data(result)
         return result
@@ -415,7 +412,7 @@ class Shrinker(object):
         # operating on a block of all zero bytes so can use non-zeroness as a
         # signpost of complexity).
         if not any(self.shrink_target.buffer) or self.incorporate_new_buffer(
-            hbytes(len(self.shrink_target.buffer))
+            bytes(len(self.shrink_target.buffer))
         ):
             return
 
@@ -632,10 +629,13 @@ class Shrinker(object):
 
         ls = self.examples_by_label[label]
 
-        i = chooser.choose(hrange(len(ls) - 1))
+        i = chooser.choose(range(len(ls) - 1))
 
         ancestor = ls[i]
 
+        if i + 1 == len(ls) or ls[i + 1].start >= ancestor.end:
+            return
+
         @self.cached(label, i)
         def descendants():
             lo = i + 1
@@ -646,10 +646,14 @@ class Shrinker(object):
                     hi = mid
                 else:
                     lo = mid
-            return ls[i + 1 : hi]
+            return [t for t in ls[i + 1 : hi] if t.length < ancestor.length]
 
         descendant = chooser.choose(descendants, lambda ex: ex.length > 0)
 
+        assert ancestor.start <= descendant.start
+        assert ancestor.end >= descendant.end
+        assert descendant.length < ancestor.length
+
         self.incorporate_new_buffer(
             self.buffer[: ancestor.start]
             + self.buffer[descendant.start : descendant.end]
@@ -709,14 +713,14 @@ class Shrinker(object):
         offset = min(ints)
         assert offset > 0
 
-        for i in hrange(len(ints)):
+        for i in range(len(ints)):
             ints[i] -= offset
 
         def reoffset(o):
             new_blocks = list(blocked)
             for i, v in zip(changed, ints):
                 new_blocks[i] = int_to_bytes(v + o, len(blocked[i]))
-            return self.incorporate_new_buffer(hbytes().join(new_blocks))
+            return self.incorporate_new_buffer(b"".join(new_blocks))
 
         Integer.shrink(offset, reoffset, random=self.random)
         self.clear_change_tracking()
@@ -767,7 +771,7 @@ class Shrinker(object):
 
                 # Between these two changed regions we now do a linear scan to
                 # check if any specific block values have changed.
-                for i in hrange(first_changed, last_changed + 1):
+                for i in range(first_changed, last_changed + 1):
                     u, v = blocks.bounds(i)
                     if i not in self.__all_changed_blocks and prev[u:v] != new[u:v]:
                         self.__all_changed_blocks.add(i)
@@ -981,7 +985,7 @@ class Shrinker(object):
         u = ex.start
         v = ex.end
         attempt = self.cached_test_function(
-            self.buffer[:u] + hbytes(v - u) + self.buffer[v:]
+            self.buffer[:u] + bytes(v - u) + self.buffer[v:]
         )
 
         if attempt is Overrun:
@@ -993,7 +997,7 @@ class Shrinker(object):
         if attempt is not self.shrink_target:
             if in_replacement.end < len(attempt.buffer) and used < ex.length:
                 self.incorporate_new_buffer(
-                    self.buffer[:u] + hbytes(used) + self.buffer[v:]
+                    self.buffer[:u] + bytes(used) + self.buffer[v:]
                 )
         return self.examples[ex.index].trivial
 
@@ -1040,6 +1044,7 @@ class Shrinker(object):
             if a < 0 or b > len(group):
                 return False
             regions = []
+
             for e in group[a:b]:
                 t = (e.start, e.end, replacement)
                 if not regions or t[0] >= regions[-1][1]:
@@ -1214,7 +1219,7 @@ class Shrinker(object):
 
         ex = self.examples[
             chooser.choose(
-                hrange(first_example_after_block, len(self.examples)),
+                range(first_example_after_block, len(self.examples)),
                 lambda i: self.examples[i].length > 0,
             )
         ]
@@ -1284,7 +1289,7 @@ class Shrinker(object):
             if d < 0:
                 return False
 
-            if self.consider_new_buffer(hbytes([d if b == c else b for b in buf])):
+            if self.consider_new_buffer(bytes([d if b == c else b for b in buf])):
                 if d <= 1:
                     # For small values of d if this succeeds we take this
                     # as evidence that it is worth doing a a bulk replacement
@@ -1303,7 +1308,7 @@ class Shrinker(object):
                             return c - k <= b <= c and d < b
 
                         return self.consider_new_buffer(
-                            hbytes([d if should_replace_byte(b) else b for b in buf])
+                            bytes([d if should_replace_byte(b) else b for b in buf])
                         )
 
                     find_integer(replace_range)
@@ -1360,7 +1365,7 @@ class Shrinker(object):
         if i + len(description) > len(original.blocks) or i < 0:
             return False
         attempt = bytearray(original.buffer)
-        for _ in hrange(repeats):
+        for _ in range(repeats):
             for k, d in reversed(list(enumerate(description))):
                 j = i + k
                 u, v = original.blocks[j].bounds
@@ -1404,7 +1409,7 @@ def block_program(description):
             """Adaptively attempt to run the block program at the current
             index. If this successfully applies the block program ``k`` times
             then this runs in ``O(log(k))`` test function calls."""
-            i = chooser.choose(hrange(len(self.shrink_target.blocks) - n))
+            i = chooser.choose(range(len(self.shrink_target.blocks) - n))
             # First, run the block program at the chosen index. If this fails,
             # don't do any extra work, so that failure is as cheap as possible.
             if not self.run_block_program(i, description, original=self.shrink_target):
@@ -1442,7 +1447,7 @@ def block_program(description):
 
 
 @attr.s(slots=True, eq=False)
-class ShrinkPass(object):
+class ShrinkPass:
     run_with_chooser = attr.ib()
     index = attr.ib()
     shrinker = attr.ib()
