@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,15 +9,11 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
-
-"""This module implements various useful common functions for shrinking tasks.
-"""
+"""This module implements various useful common functions for shrinking tasks."""
 
 
 def find_integer(f):
@@ -60,7 +54,7 @@ def find_integer(f):
     return lo
 
 
-class Shrinker(object):
+class Shrinker:
     """A Shrinker object manages a single value and a predicate it should
     satisfy, and attempts to improve it in some direction, making it smaller
     and simpler."""
@@ -98,7 +92,6 @@ class Shrinker(object):
         Convenience function for children that doesn't require messing
         with the signature of init.
         """
-        pass
 
     def delegate(self, other_class, convert_to, convert_from, **kwargs):
         """Delegates shrinking to another shrinker class, by converting the
@@ -107,7 +100,7 @@ class Shrinker(object):
             other_class,
             convert_to(self.current),
             lambda v: self.consider(convert_from(v)),
-            **kwargs
+            **kwargs,
         )
 
     def call_shrinker(self, other_class, initial, predicate, **kwargs):
@@ -198,7 +191,7 @@ class Shrinker(object):
 
         Does nothing by default.
         """
-        pass
+        raise NotImplementedError()
 
     def short_circuit(self):
         """Possibly attempt to do some shrinking.
